"""
    SleekXMPP: The Sleek XMPP Library
    Copyright (C) 2011  Nathanael C. Fritz
    This file is part of SleekXMPP.

    See the file LICENSE for copying permission.
"""

__all__ = [
    'feature_starttls', 
    'feature_mechanisms', 
    'feature_bind', 
    'feature_session',
    'feature_rosterver'
]
